-----------------------------------------------------------------------------------
-- Maratis
-- Follow script test
-----------------------------------------------------------------------------------

-- globals
t = 0
gameState = 0
goodSoundState = 0
fushSoundState = 0
end_t = 0
level = 1
maxMonster = 64

-- get objects
OFlame = getObject("flame")
OLight = getObject("light")
OPlayer = getObject("player")
OPlayerFeetBox = getObject("player_feet_box")
OPlayerHead = getObject("player_head")
OPlayerBody = getObject("player_body")
OMonster = getObject("monster")
OMonsterHead = getObject("monster_head")
OGround = getObject("ground")
OTitle = getObject("title")
OExplo2 = getObject("explo1")
OGood = getObject("good")
OBidule = getObject("bidule")
OFush = getObject("fush")
OOver = getObject("over")
OBox01 = getObject("box01")
OBox02 = getObject("box02")
OMonsterSound = getObject("monsterSound")
OText01 = getObject("text01")
OText02 = getObject("text02")
OKeyboard = getObject("keyboard")

stopSound(OMonsterSound)

-- boxes
OBox01_pos = getPosition(OBox01)
OBox02_pos = getPosition(OBox02)
OBox01_rot = getRotation(OBox01)
OBox02_rot = getRotation(OBox02)

function getMonCount()
	count = level*16
	if count > maxMonster then
		count = maxMonster
	end
	return count
end

function wonGame()
	won = 1
	monsCount = getMonCount()
	for i=1, monsCount do
		if isActive(Monsters[i].object) then
			won = 0
		end
	end
	return won
end

function respawnBoxes()
	clearForces(OBox01)
	clearForces(OBox02)
	setPosition(OBox01, OBox01_pos)
	setPosition(OBox02, OBox02_pos)
	setRotation(OBox01, OBox01_rot)
	setRotation(OBox02, OBox02_rot)
end

function gameOver()
	level = 1
	gameState = 0
	playSound(OOver)
	monsCount = getMonCount()
	for i=1, monsCount do
		Monsters[i]:respawn(0)
	end
	
	for i=monsCount+1, maxMonster do
		deactivate(Monsters[i].object)
		deactivate(Monsters[i].head)
	end
	
	Player:respawn()
	respawnBoxes()
	stopSound(OMonsterSound)
	setSoundGain(OMonsterSound, 0)
	activate(OText01)
	activate(OText02)
	activate(OKeyboard)
end

-- tools
function getLength2d(vec)
	return math.sqrt(vec[1]*vec[1] + vec[2]*vec[2])
end

function normalize2d(vec)
	l = getLength2d(vec)
	if l > 0 then
		vec[1] = vec[1] / l
		vec[2] = vec[2] / l
	end
	return vec
end

function lookAt(object, dir, rotSpeed)
	rot = getRotation(object)

	zAngle = math.rad(rot[3])
	sinZ = math.sin(zAngle)
	cosZ = math.cos(zAngle)

	YDir = normalize2d({sinZ, -cosZ})
	XDir = {-YDir[2], YDir[1]}

	ori = (dir[1]*XDir[1] + dir[2]*XDir[2])
	angle = math.acos(dir[1]*YDir[1] + dir[2]*YDir[2])

	if ori < 0 then
		angle = - angle
	end

	rotate(object, {0, 0, 1}, math.deg(angle) * rotSpeed)
end


-- behaviors
BFlame = {}
BFlame.__index = BFlame
function BFlame.create(object, light)
	local newObj = {}
	setmetatable(newObj, BFlame)
	newObj.object = object
	newObj.light = light
	newObj.scale = getScale(object)
	newObj.flamePower = 1.0
	return newObj
end
function BFlame:update()

	-- won
	if gameState == 1 and wonGame() == 1 then
		gameState = 3
		level = level + 1
		end_t = 0
	-- game over
	elseif self.flamePower < 0.0125 and gameState == 1 then
		gameState = 2
		end_t = 0
		return
	end
	
	-- flame
	flameSpeed = 0.99995
	self.flamePower = math.pow(self.flamePower*flameSpeed, 1.001)
	if self.flamePower < 0.35 and self.flamePower > 0.0125 then
		self.flamePower = self.flamePower - 0.0125
	end
		
	si = math.sin(t*0.5)
	co = math.cos(t*0.5)
	si2 = math.sin(t*0.35)
	setScale(self.object, self.scale * vec3(0.9 + co*0.1, 0.9 + si*0.1, 1.0) * self.flamePower)
	setLightIntensity(self.light, (4 + si2*0.75)*self.flamePower)
end

BPlayer = {}
BPlayer.__index = BPlayer
function BPlayer.create()
	local newObj = {}
	setmetatable(newObj, BPlayer)
	newObj.object = OPlayer
	newObj.headPos = getPosition(OPlayerHead)
	newObj.bodyPos = getPosition(OPlayerBody)
	newObj.flamePos = getPosition(OFlame)
	newObj.soundt = 0
	newObj.spawnPos = getPosition(newObj.object)
	return newObj
end
function BPlayer:respawn()
	clearForces(self.object)
	setPosition(self.object, self.spawnPos)
	changeAnimation(OPlayerHead, 0)
end
function BPlayer:update()
	force = 1.5
	coll = getNumCollisions(OPlayerFeetBox)
	
	if coll < 2 then
		setLinearDamping(self.object, 0.01)
		setPosition(OPlayerHead, self.headPos)
		setPosition(OPlayerBody, self.bodyPos)
	else
		setLinearDamping(self.object, 0.9)
	
		vec = vec3(0, 0, 0)
		if isKeyPressed("UP") then
			vec = vec + vec3(0, 1, 0)
			changeAnimation(OPlayerHead, 3)
		end
		if isKeyPressed("DOWN") then
			vec = vec + vec3(0, -1, 0)
			changeAnimation(OPlayerHead, 0)
		end
		if isKeyPressed("RIGHT") then
			vec = vec + vec3(1, 0, 0)
			changeAnimation(OPlayerHead, 2)
		end
		if isKeyPressed("LEFT") then
			vec = vec + vec3(-1, 0, 0)
			changeAnimation(OPlayerHead, 1)
		end
		
		l = length(vec)
		if l > 0 then
			addCentralForce(self.object, (vec/l)*force)
			
			si = math.sin(t*0.5)
			si2 = math.sin(t*0.5+1)
			si3 = math.sin(t*0.5+2.5)
			setPosition(OPlayerHead, self.headPos + vec3(0.0, 0.0, si*1.5))
			setPosition(OPlayerBody, self.bodyPos + vec3(0.0, 0.0, si2*1.5))
			setPosition(OFlame, self.flamePos + vec3(0.0, 0.0, si3*1.5))
			
			lookAt(OLight, normalize2d({vec[1], vec[2]}), 0.1)
		else
			setPosition(OPlayerHead, self.headPos)
			setPosition(OPlayerBody, self.bodyPos)
		end
	end
	
	-- sound
	if self.soundt > 0 then
		self.soundt = self.soundt + 1
		if self.soundt > 240 then
			self.soundt = 0
		end
	end
	
	-- game over
	pos = getPosition(self.object)
	if pos[3] < 0 then
		gameState = 2
		end_t = 0
	end
end

BMonster = {}
BMonster.__index = BMonster
function BMonster.create()
	local newObj = {}
	setmetatable(newObj, BMonster)
	
	offset = vec3(math.random(), math.random(), 0.5)*2.0 - 1.0
	pos = getPosition(OMonster) + offset*20.0

	newObj.object = getClone(OMonster)
	newObj.head = getClone(OMonsterHead)
	setParent(newObj.head, newObj.object)
	
	setPosition(newObj.object, pos)
	
	s = (math.random()*2 - 1)*0.075 + 0.1
	setScale(newObj.head, getScale(OMonster) + vec3(s, s, s))
	
	newObj.headPos = getPosition(newObj.head)
	newObj.rand = math.random()
	newObj.prevPos = getPosition(newObj.object)
	newObj.on = 0
	newObj.t = 0
	newObj.spawnPos = newObj.prevPos
	
	return newObj
end
function BMonster:respawn(z)
	activate(self.object)
	activate(self.head)
	clearForces(self.object)
	setPosition(self.object, self.spawnPos + vec3(0, 0, z))
	changeAnimation(self.head, 1)
	self.on = 0
	self.t = 0
	self.prevPos = self.spawnPos
end
function BMonster:update()
	pos = getPosition(self.object)
	groundColl = isCollisionBetween(self.object, OGround)
	
	if groundColl then
		self.on = 1
	end

	if self.on == 1 then
		self.t = self.t+1
	end

	if self.t > 0 and groundColl then
		force = 0.15 + 0.15*(1.0 - Flame.flamePower)
		dir = getPosition(OPlayer) - getPosition(self.object)
		l = length(dir)
		r = 25 * Flame.flamePower
		
		--if l < 40 then
			if l > r then
					addCentralForce(self.object, normalize(dir)*force)
			else
				addCentralForce(self.object, -normalize(dir)*force)
			end
			
			if math.abs(dir[1]) > math.abs(dir[2]) then
				if dir[1] > 0 then
					changeAnimation(self.head, 3)
				else
					changeAnimation(self.head, 2)
				end
			else
				if dir[2] > 0 then
					changeAnimation(self.head, 1)
				else
					changeAnimation(self.head, 0)
				end
			end
			
			speed = length(pos - self.prevPos)
			si = math.abs(math.sin(t*0.25 + self.rand*10.0))
			setPosition(self.head, self.headPos + vec3(0.0, 0.0, si*speed*10.0))
		--else
		--	changeAnimation(self.head, 1)
		--end
	end
	
	if pos[3] < -50 and isActive(self.object) then
		deactivate(self.object)
		if goodSoundState == 0 then
			goodSoundState = 1
			playSound(OGood)
		end
	end
	
	self.prevPos = pos
end


-- init
Player = BPlayer.create()
Flame = BFlame.create(OFlame, OLight)

math.randomseed(3)
Monsters = {}
for i=1, maxMonster do
	Monsters[i] = BMonster.create()
end

deactivate(OMonster)
deactivate(OMonsterHead)

moCount = getMonCount()
for i=moCount+1, maxMonster do
	deactivate(Monsters[i].object)
	deactivate(Monsters[i].head)
end

titleEndPos = getPosition(OTitle)


-- update
function updateSoundState(state, duration)
	if state > 0 then
		state = state + 1
		if state > duration then
			state = 0
		end
	end
	return state
end

function onSceneUpdate()

	monsCount = getMonCount()

	if gameState == 1 then
	
		-- title
		if isActive(OTitle) then
			dir = vec3(titleEndPos[1], titleEndPos[2], titleEndPos[3] + 200)
			translate(OTitle, dir*0.005)
		end
	
		-- behaviors
		Player:update()
		Flame:update()
		
		if gameState == 1 then
			for i=1, monsCount do
				Monsters[i]:update()
			end
		end
		
		-- sound
		goodSoundState = updateSoundState(goodSoundState, 120)
		fushSoundState = updateSoundState(fushSoundState, 120)
		
		mgain = getSoundGain(OMonsterSound)
		if mgain < 0.65 then
			setSoundGain(OMonsterSound, mgain+0.01)
		end
		
	elseif gameState == 2 then -- over
	
		for i=1, monsCount do
			Monsters[i]:update()
		end
		
		end_t = end_t + 1
		if end_t > 60 then
			gameOver()
		end
		
	elseif gameState == 3 then -- won
	
		stopSound(OMonsterSound)
		Flame:update()
		Player:update()
		end_t = end_t + 1
		
		Flame.flamePower = Flame.flamePower + 0.35/450.0
		if end_t > 450 or Flame.flamePower > 1.0 then
			for i=1, monsCount do
				Monsters[i]:respawn(10 + math.random()*20)
			end
			gameState = 1
			playSound(OMonsterSound)
			if Flame.flamePower > 1.0 then
				Flame.flamePower = 1
			end
		end
		
	else
		-- title
		vec = getRotatedVector(OTitle, vec3(0, 0, 1))*2
		setPosition(OTitle, titleEndPos + vec * math.sin(t*0.085))
		
		si = math.sin(t*0.2)
		co = (1.0 - si)
		setScale(OText02, vec3(0.9+si*0.025, 0.68, 0.1+co*0.005))
		
		-- flame
		Flame:update()
		Flame.flamePower = 1
		
		if isKeyPressed("ENTER") then
			gameState = 1
			playSound(OExplo2)
			playSound(OMonsterSound)
			deactivate(OText01)
			deactivate(OText02)
			deactivate(OKeyboard)
		end
		
	end
	
	t = t+1
	
end
